@@ -343,8 +343,9 @@ module Cri
           explicitly_no_params?,
         )
         handle_errors_while { parser.run }
-        local_opts  = parser.options
+        local_opts = parser.options
         global_opts = parent_opts.merge(parser.options)
+        global_opts = add_defaults(global_opts)
 
         # Handle options
         handle_options(local_opts)
@@ -359,6 +360,14 @@ module Cri
       block.call(global_opts, args, self)
     end
 
+    def all_opt_defns
+      if supercommand
+        supercommand.all_opt_defns | option_definitions
+      else
+        option_definitions
+      end
+    end
+
     # @return [String] The help text for this command
     #
     # @option params [Boolean] :verbose true if the help output should be
@@ -366,8 +375,8 @@ module Cri
     #
     # @option params [IO] :io ($stdout) the IO the help text is intended for.
     #   This influences the decision to enable/disable colored output.
-    def help(params = {})
-      HelpRenderer.new(self, params).render
+    def help(**params)
+      HelpRenderer.new(self, **params).render
     end
 
     # Compares this command's name to the other given command's name.
@@ -428,5 +437,23 @@ module Cri
       warn "#{name}: #{e.message}"
       raise CriExitException.new(is_error: true)
     end
+
+    def add_defaults(options)
+      all_opt_defns_by_key =
+        all_opt_defns.each_with_object({}) do |opt_defn, hash|
+          key = (opt_defn.long || opt_defn.short).to_sym
+          hash[key] = opt_defn
+        end
+
+      new_options = Hash.new do |hash, key|
+        hash.fetch(key) { all_opt_defns_by_key[key]&.default }
+      end
+
+      options.each do |key, value|
+        new_options[key] = value
+      end
+
+      new_options
+    end
   end
 end
