@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'helper'
 
 module Cri
@@ -16,6 +18,7 @@ module Cri
         optional  :c, :ccc, 'opt c'
         flag      :d, :ddd, 'opt d'
         forbidden :e, :eee, 'opt e'
+        required  :t, :transform, 'opt t', transform: method(:Integer)
 
         run do |opts, args, c|
           $stdout.puts "Awesome #{c.name}!"
@@ -175,7 +178,7 @@ module Cri
       end
 
       assert_equal [], lines(out)
-      assert_equal ['moo: illegal option -- z'], lines(err)
+      assert_equal ['moo: unrecognised option -- z'], lines(err)
     end
 
     def test_invoke_simple_with_illegal_opt_no_exit
@@ -184,7 +187,28 @@ module Cri
       end
 
       assert_equal [], lines(out)
-      assert_equal ['moo: illegal option -- z'], lines(err)
+      assert_equal ['moo: unrecognised option -- z'], lines(err)
+    end
+
+    def test_invoke_simple_with_invalid_value_for_opt
+      out, err = capture_io_while do
+        err = assert_raises SystemExit do
+          simple_cmd.run(%w[-t nope])
+        end
+        assert_equal 1, err.status
+      end
+
+      assert_equal [], lines(out)
+      assert_equal ['moo: invalid value "nope" for --transform option'], lines(err)
+    end
+
+    def test_invoke_simple_with_invalid_value_for_opt_no_exit
+      out, err = capture_io_while do
+        simple_cmd.run(%w[-t nope], {}, hard_exit: false)
+      end
+
+      assert_equal [], lines(out)
+      assert_equal ['moo: invalid value "nope" for --transform option'], lines(err)
     end
 
     def test_invoke_simple_with_opt_with_block
@@ -455,8 +479,8 @@ module Cri
 
       # Check option definitions
       assert_equal 1, cmd.option_definitions.size
-      opt_def = cmd.option_definitions.to_a[0]
-      assert_equal 'help', opt_def[:long]
+      opt_defn = cmd.option_definitions.to_a[0]
+      assert_equal 'help', opt_defn.long
 
       # Check subcommand
       assert_equal 1,      cmd.subcommands.size
@@ -525,7 +549,7 @@ module Cri
     end
 
     def test_hidden_commands_multiple
-      cmd    = nested_cmd
+      cmd = nested_cmd
 
       subcmd = simple_cmd
       cmd.add_command subcmd
@@ -568,14 +592,14 @@ module Cri
       cmd = Cri::Command.define do
         name 'moo'
         run do |_opts, args|
-          puts "args=#{args.join(',')} args.raw=#{args.raw.join(',')}"
+          puts "args=#{args.join(',')}"
         end
       end
 
       out, _err = capture_io_while do
         cmd.run(%w[foo -- bar])
       end
-      assert_equal "args=foo,bar args.raw=foo,--,bar\n", out
+      assert_equal "args=foo,bar\n", out
     end
 
     def test_run_without_block
@@ -593,7 +617,7 @@ module Cri
         name 'moo'
         runner(Class.new(Cri::CommandRunner) do
           def run
-            puts "args=#{arguments.join(',')} args.raw=#{arguments.raw.join(',')}"
+            puts "args=#{arguments.join(',')}"
           end
         end)
       end
@@ -601,7 +625,7 @@ module Cri
       out, _err = capture_io_while do
         cmd.run(%w[foo -- bar])
       end
-      assert_equal "args=foo,bar args.raw=foo,--,bar\n", out
+      assert_equal "args=foo,bar\n", out
     end
 
     def test_compare
@@ -673,5 +697,308 @@ module Cri
 
       assert_equal "opts={:aaa=>\"test\"} args=--test,value\n", out
     end
+
+    def test_wrong_number_of_args
+      cmd = Cri::Command.define do
+        name 'publish'
+        param :filename
+      end
+
+      out, err = capture_io_while do
+        err = assert_raises SystemExit do
+          cmd.run([])
+        end
+        assert_equal 1, err.status
+      end
+
+      assert_equal [], lines(out)
+      assert_equal ['publish: incorrect number of arguments given: expected 1, but got 0'], lines(err)
+    end
+
+    def test_no_params_zero_args
+      dsl = Cri::CommandDSL.new
+      dsl.instance_eval do
+        name        'moo'
+        usage       'dunno whatever'
+        summary     'does stuff'
+        description 'This command does a lot of stuff.'
+        no_params
+
+        run do |_opts, args|
+        end
+      end
+      command = dsl.command
+
+      command.run([])
+    end
+
+    def test_no_params_one_arg
+      dsl = Cri::CommandDSL.new
+      dsl.instance_eval do
+        name        'moo'
+        usage       'dunno whatever'
+        summary     'does stuff'
+        description 'This command does a lot of stuff.'
+        no_params
+
+        run do |_opts, args|
+        end
+      end
+      command = dsl.command
+
+      out, err = capture_io_while do
+        err = assert_raises SystemExit do
+          command.run(['a'])
+        end
+        assert_equal 1, err.status
+      end
+
+      assert_equal [], lines(out)
+      assert_equal ['moo: incorrect number of arguments given: expected 0, but got 1'], lines(err)
+    end
+
+    def test_load_file
+      Dir.mktmpdir('foo') do |dir|
+        filename = "#{dir}/moo.rb"
+        File.write(filename, <<~CMD)
+          name        'moo'
+          usage       'dunno whatever'
+          summary     'does stuff'
+          description 'This command does a lot of stuff.'
+          no_params
+
+          run do |_opts, args|
+          end
+        CMD
+
+        cmd = Cri::Command.load_file(filename)
+        assert_equal('moo', cmd.name)
+      end
+    end
+
+    def test_load_file_infer_name_false
+      Dir.mktmpdir('foo') do |dir|
+        filename = "#{dir}/moo.rb"
+        File.write(filename, <<~CMD)
+          usage       'dunno whatever'
+          summary     'does stuff'
+          description 'This command does a lot of stuff.'
+          no_params
+
+          run do |_opts, args|
+          end
+        CMD
+
+        cmd = Cri::Command.load_file(filename)
+        assert_equal(nil, cmd.name)
+      end
+    end
+
+    def test_load_file_infer_name
+      Dir.mktmpdir('foo') do |dir|
+        filename = "#{dir}/moo.rb"
+        File.write(filename, <<~CMD)
+          usage       'dunno whatever'
+          summary     'does stuff'
+          description 'This command does a lot of stuff.'
+          no_params
+
+          run do |_opts, args|
+          end
+        CMD
+
+        cmd = Cri::Command.load_file(filename, infer_name: true)
+        assert_equal('moo', cmd.name)
+      end
+    end
+
+    def test_load_file_infer_name_double
+      Dir.mktmpdir('foo') do |dir|
+        filename = "#{dir}/moo.rb"
+        File.write(filename, <<~CMD)
+          name        'oink'
+          usage       'dunno whatever'
+          summary     'does stuff'
+          description 'This command does a lot of stuff.'
+          no_params
+
+          run do |_opts, args|
+          end
+        CMD
+
+        cmd = Cri::Command.load_file(filename, infer_name: true)
+        assert_equal('moo', cmd.name)
+      end
+    end
+
+    def test_required_args_with_dash_h
+      dsl = Cri::CommandDSL.new
+      dsl.instance_eval do
+        name        'moo'
+        usage       'dunno whatever'
+        summary     'does stuff'
+        description 'This command does a lot of stuff.'
+
+        param :foo
+
+        option :h, :help, 'show help' do
+          $helped = true
+          exit 0
+        end
+      end
+      command = dsl.command
+
+      $helped = false
+      out, err = capture_io_while do
+        assert_raises SystemExit do
+          command.run(['-h'])
+        end
+      end
+      assert $helped
+      assert_equal [], lines(out)
+      assert_equal [], lines(err)
+    end
+
+    def test_propagate_options_two_levels_down
+      cmd_a = Cri::Command.define do
+        name 'a'
+        flag :t, :test, 'test'
+      end
+
+      cmd_b = cmd_a.define_command('b') do
+      end
+
+      cmd_b.define_command('c') do
+        run do |opts, _args|
+          puts "test? #{opts[:test].inspect}!"
+        end
+      end
+
+      # test -t last
+      out, err = capture_io_while do
+        cmd_a.run(%w[b c -t])
+      end
+      assert_equal ['test? true!'], lines(out)
+      assert_equal [], lines(err)
+
+      # test -t mid
+      out, err = capture_io_while do
+        cmd_a.run(%w[b -t c])
+      end
+      assert_equal ['test? true!'], lines(out)
+      assert_equal [], lines(err)
+
+      # test -t first
+      out, err = capture_io_while do
+        cmd_a.run(%w[-t b c])
+      end
+      assert_equal ['test? true!'], lines(out)
+      assert_equal [], lines(err)
+    end
+
+    def test_flag_defaults_to_false
+      cmd = Cri::Command.define do
+        name 'a'
+        option :f, :force2, 'push with force', argument: :forbidden
+
+        run do |opts, _args, _cmd|
+          puts "Force? #{opts[:force2].inspect}! Key present? #{opts.key?(:force2)}!"
+        end
+      end
+
+      out, err = capture_io_while do
+        cmd.run(%w[])
+      end
+      assert_equal ['Force? false! Key present? false!'], lines(out)
+      assert_equal [], lines(err)
+    end
+
+    def test_required_option_defaults_to_given_value
+      cmd = Cri::Command.define do
+        name 'a'
+        option :a, :animal, 'specify animal', argument: :required, default: 'cow'
+
+        run do |opts, _args, _cmd|
+          puts "Animal = #{opts[:animal]}! Key present? #{opts.key?(:animal)}!"
+        end
+      end
+
+      out, err = capture_io_while do
+        cmd.run(%w[])
+      end
+      assert_equal ['Animal = cow! Key present? false!'], lines(out)
+      assert_equal [], lines(err)
+    end
+
+    def test_optional_option_defaults_to_given_value
+      cmd = Cri::Command.define do
+        name 'a'
+        option :a, :animal, 'specify animal', argument: :optional, default: 'cow'
+
+        run do |opts, _args, _cmd|
+          puts "Animal = #{opts[:animal]}"
+        end
+      end
+
+      out, err = capture_io_while do
+        cmd.run(%w[])
+      end
+      assert_equal ['Animal = cow'], lines(out)
+      assert_equal [], lines(err)
+    end
+
+    def test_required_option_defaults_to_given_value_with_transform
+      cmd = Cri::Command.define do
+        name 'a'
+        option :a, :animal, 'specify animal', argument: :required, transform: ->(a) { a.upcase }, default: 'cow'
+
+        run do |opts, _args, _cmd|
+          puts "Animal = #{opts[:animal]}"
+        end
+      end
+
+      out, err = capture_io_while do
+        cmd.run(%w[])
+      end
+      assert_equal ['Animal = cow'], lines(out)
+      assert_equal [], lines(err)
+    end
+
+    def test_option_definitions_are_not_shared_across_commands
+      root_cmd = Cri::Command.define do
+        name 'root'
+        option :r, :rrr, 'Rrr!', argument: :required
+      end
+
+      subcmd_a = root_cmd.define_command do
+        name 'a'
+        option :a, :aaa, 'Aaa!', argument: :required
+
+        run do |_opts, _args, cmd|
+          puts cmd.all_opt_defns.map(&:long).sort.join(',')
+        end
+      end
+
+      subcmd_b = root_cmd.define_command do
+        name 'b'
+        option :b, :bbb, 'Bbb!', argument: :required
+
+        run do |_opts, _args, cmd|
+          puts cmd.all_opt_defns.map(&:long).sort.join(',')
+        end
+      end
+
+      out, err = capture_io_while do
+        subcmd_a.run(%w[])
+      end
+      assert_equal ['aaa,rrr'], lines(out)
+      assert_equal [], lines(err)
+
+      out, err = capture_io_while do
+        subcmd_b.run(%w[])
+      end
+      assert_equal ['bbb,rrr'], lines(out)
+      assert_equal [], lines(err)
+    end
   end
 end
