@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'helper'
 
 module Cri
@@ -39,15 +41,15 @@ module Cri
       expected_option_definitions =
         Set.new(
           [
-            { short: 'a', long: 'aaa', desc: 'opt a', argument: :optional,  multiple: true, hidden: false, block: nil, default: nil },
-            { short: 'b', long: 'bbb', desc: 'opt b', argument: :required,  multiple: false, hidden: false, block: nil, default: nil },
-            { short: 'c', long: 'ccc', desc: 'opt c', argument: :optional,  multiple: false, hidden: false, block: nil, default: nil },
-            { short: 'd', long: 'ddd', desc: 'opt d', argument: :forbidden, multiple: false, hidden: false, block: nil, default: nil },
-            { short: 'e', long: 'eee', desc: 'opt e', argument: :forbidden, multiple: false, hidden: false, block: nil, default: nil },
-            { short: 'f', long: 'fff', desc: 'opt f', argument: :forbidden, multiple: false, hidden: true,  block: nil, default: nil },
+            { short: 'a', long: 'aaa', desc: 'opt a', argument: :optional,  multiple: true,  hidden: false, block: nil, default: nil,   transform: nil },
+            { short: 'b', long: 'bbb', desc: 'opt b', argument: :required,  multiple: false, hidden: false, block: nil, default: nil,   transform: nil },
+            { short: 'c', long: 'ccc', desc: 'opt c', argument: :optional,  multiple: false, hidden: false, block: nil, default: nil,   transform: nil },
+            { short: 'd', long: 'ddd', desc: 'opt d', argument: :forbidden, multiple: false, hidden: false, block: nil, default: false, transform: nil },
+            { short: 'e', long: 'eee', desc: 'opt e', argument: :forbidden, multiple: false, hidden: false, block: nil, default: false, transform: nil },
+            { short: 'f', long: 'fff', desc: 'opt f', argument: :forbidden, multiple: false, hidden: true,  block: nil, default: false, transform: nil },
           ],
         )
-      actual_option_definitions = Set.new(command.option_definitions)
+      actual_option_definitions = Set.new(command.option_definitions.map(&:to_h))
       assert_equal expected_option_definitions, actual_option_definitions
     end
 
@@ -78,11 +80,11 @@ module Cri
       expected_option_definitions =
         Set.new(
           [
-            { short: 's', long: nil, desc: 'short', argument: :forbidden, multiple: false, hidden: false, block: nil, default: nil },
-            { short: nil, long: 'long', desc: 'long', argument: :forbidden, multiple: false, hidden: false, block: nil, default: nil },
+            { short: 's', long: nil,    desc: 'short', argument: :forbidden, multiple: false, hidden: false, block: nil, default: false, transform: nil },
+            { short: nil, long: 'long', desc: 'long',  argument: :forbidden, multiple: false, hidden: false, block: nil, default: false, transform: nil },
           ],
         )
-      actual_option_definitions = Set.new(command.option_definitions)
+      actual_option_definitions = Set.new(command.option_definitions.map(&:to_h))
       assert_equal expected_option_definitions, actual_option_definitions
     end
 
@@ -102,12 +104,12 @@ module Cri
       expected_option_definitions =
         Set.new(
           [
-            { short: 'f', long: 'flag',     desc: 'flag', argument: :forbidden, multiple: true, hidden: false, block: nil, default: nil },
-            { short: 'r', long: 'required', desc: 'req',  argument: :required,  multiple: true, hidden: false, block: nil, default: nil },
-            { short: 'o', long: 'optional', desc: 'opt',  argument: :optional,  multiple: true, hidden: false, block: nil, default: nil },
+            { short: 'f', long: 'flag',     desc: 'flag', argument: :forbidden, multiple: true, hidden: false, block: nil, default: false, transform: nil },
+            { short: 'r', long: 'required', desc: 'req',  argument: :required,  multiple: true, hidden: false, block: nil, default: nil,   transform: nil },
+            { short: 'o', long: 'optional', desc: 'opt',  argument: :optional,  multiple: true, hidden: false, block: nil, default: nil,   transform: nil },
           ],
         )
-      actual_option_definitions = Set.new(command.option_definitions)
+      actual_option_definitions = Set.new(command.option_definitions.map(&:to_h))
       assert_equal expected_option_definitions, actual_option_definitions
     end
 
@@ -127,15 +129,23 @@ module Cri
       expected_option_definitions =
         Set.new(
           [
-            { short: 'f', long: 'flag',     desc: 'flag', argument: :forbidden, multiple: false, hidden: true, block: nil, default: nil },
-            { short: 'r', long: 'required', desc: 'req',  argument: :required,  multiple: false, hidden: true, block: nil, default: nil },
-            { short: 'o', long: 'optional', desc: 'opt',  argument: :optional,  multiple: false, hidden: true, block: nil, default: nil },
+            { short: 'f', long: 'flag',     desc: 'flag', argument: :forbidden, multiple: false, hidden: true, block: nil, default: false, transform: nil },
+            { short: 'r', long: 'required', desc: 'req',  argument: :required,  multiple: false, hidden: true, block: nil, default: nil,   transform: nil },
+            { short: 'o', long: 'optional', desc: 'opt',  argument: :optional,  multiple: false, hidden: true, block: nil, default: nil,   transform: nil },
           ],
         )
-      actual_option_definitions = Set.new(command.option_definitions)
+      actual_option_definitions = Set.new(command.option_definitions.map(&:to_h))
       assert_equal expected_option_definitions, actual_option_definitions
     end
 
+    def test_raises_on_unrecognized_option
+      # Define
+      dsl = Cri::CommandDSL.new
+      assert_raises ArgumentError do
+        dsl.option :s, :long, 'desc', unrecognized: true
+      end
+    end
+
     def test_required_short_and_long
       # Define
       dsl = Cri::CommandDSL.new
@@ -222,7 +232,7 @@ module Cri
     def test_runner
       # Define
       dsl = Cri::CommandDSL.new
-      dsl.instance_eval <<-EOS
+      dsl.instance_eval(<<-CMD, __FILE__, __LINE__ + 1)
         class Cri::CommandDSLTestCaseCommandRunner < Cri::CommandRunner
           def run
             $did_it_work = arguments[0]
@@ -230,7 +240,7 @@ module Cri
         end
 
         runner Cri::CommandDSLTestCaseCommandRunner
-  EOS
+      CMD
       command = dsl.command
 
       # Check
@@ -238,5 +248,89 @@ module Cri
       command.run(%w[certainly])
       assert_equal 'certainly', $did_it_work
     end
+
+    def test_params
+      # Define
+      dsl = Cri::CommandDSL.new
+      dsl.instance_eval do
+        name        'moo'
+        usage       'dunno whatever'
+        summary     'does stuff'
+        description 'This command does a lot of stuff.'
+
+        param :foo
+        param :bar
+        param :qux
+
+        run do |_opts, args|
+          $args_num = { foo: args[0], bar: args[1], qux: args[2] }
+          $args_sym = { foo: args[:foo], bar: args[:bar], qux: args[:qux] }
+        end
+      end
+      command = dsl.command
+
+      # Run
+      $args_num = '???'
+      $args_sym = '???'
+      command.run(%w[a b c])
+      assert_equal({ foo: 'a', bar: 'b', qux: 'c' }, $args_num)
+      assert_equal({ foo: 'a', bar: 'b', qux: 'c' }, $args_sym)
+    end
+
+    def test_params_transform
+      # Define
+      dsl = Cri::CommandDSL.new
+      dsl.instance_eval do
+        name        'moo'
+        usage       'dunno whatever'
+        summary     'does stuff'
+        description 'This command does a lot of stuff.'
+
+        param :foo, transform: ->(a) { a.upcase }
+
+        run do |_opts, args|
+          $args_num = { foo: args[0] }
+          $args_sym = { foo: args[:foo] }
+        end
+      end
+      command = dsl.command
+
+      # Run
+      $args_num = '???'
+      $args_sym = '???'
+      command.run(%w[abc])
+      assert_equal({ foo: 'ABC' }, $args_num)
+      assert_equal({ foo: 'ABC' }, $args_sym)
+    end
+
+    def test_no_params_with_one_param_specified
+      dsl = Cri::CommandDSL.new
+      err = assert_raises Cri::CommandDSL::AlreadySpecifiedWithParams do
+        dsl.instance_eval do
+          name        'moo'
+          usage       'dunno whatever'
+          summary     'does stuff'
+          description 'This command does a lot of stuff.'
+          param :oink
+          no_params
+        end
+      end
+      assert_equal('Attempted to declare the command "moo" as taking no parameters, but some parameters are already declared for this command. Suggestion: remove the #no_params call.', err.message)
+    end
+
+    def test_one_param_with_no_params_specified
+      dsl = Cri::CommandDSL.new
+      err = assert_raises Cri::CommandDSL::AlreadySpecifiedAsNoParams do
+        dsl.instance_eval do
+          name        'moo'
+          usage       'dunno whatever'
+          summary     'does stuff'
+          description 'This command does a lot of stuff.'
+          no_params
+          param :oink
+        end
+      end
+      assert_equal('Attempted to specify a parameter :oink to the command "moo", which is already specified as taking no params. Suggestion: remove the #no_params call.', err.message)
+    end
   end
 end
